/*
base custom mode
 */
// This only defines high-level behaviour of the Mode like folding etc.
ace.define(
	'ace/mode/gwql',
	['require', 'exports', 'ace/lib/oop', 'ace/mode/text', 'ace/mode/custom_highlight_rules'],
	(acequire, exports) => {
		const oop = acequire('ace/lib/oop');
		const TextMode = acequire('ace/mode/text').Mode;
		const GwqlHighlightRules = acequire('ace/mode/gwql_highlight_rules').GwqlHighlightRules;

		var Mode = function () {
			this.HighlightRules = GwqlHighlightRules;
		};

		oop.inherits(Mode, TextMode); // ACE's way of doing inheritance

		(function () {
			this.$id = 'ace/mode/gwql';
		}.call(Mode.prototype));

		exports.Mode = Mode; // eslint-disable-line no-param-reassign
	},
);

// This is where we really create the highlighting rules
ace.define(
	'ace/mode/gwql_highlight_rules',
	['require', 'exports', 'ace/lib/oop', 'ace/mode/text_highlight_rules'],
	(acequire, exports) => {
		const oop = acequire('ace/lib/oop');
		const TextHighlightRules = acequire('ace/mode/text_highlight_rules').TextHighlightRules;

		const GwqlHighlightRules = function GwqlHighlightRules() {
			var keywords = 'tag';

			var builtinConstants = 'true|false';

			var builtinFunctions = 'avg|count|grep|chart|table|text|raw';

			var dataTypes =
				'int|numeric|decimal|date|varchar|char|bigint|float|double|bit|binary|text|set|timestamp|' +
				'money|real|number|integer';

			var keywordMapper = this.createKeywordMapper(
				{
					'support.function': builtinFunctions,
					keyword: keywords,
					'constant.language': builtinConstants,
					'storage.type': dataTypes,
				},
				'identifier',
				true,
			);

			this.$rules = {
				start: [
					{
						token: 'comment',
						regex: '--.*$',
					},
					{
						token: 'comment',
						start: '/\\*',
						end: '\\*/',
					},
					{
						token: 'string', // " string
						regex: '".*?"',
					},
					{
						token: 'string', // ' string
						regex: "'.*?'",
					},
					{
						token: 'string', // ` string (apache drill)
						regex: '`.*?`',
					},
					{
						token: 'constant.numeric', // float
						regex: '[+-]?\\d+(?:(?:\\.\\d*)?(?:[eE][+-]?\\d+)?)?\\b',
					},
					{
						token: keywordMapper,
						regex: '[a-zA-Z_$][a-zA-Z0-9_$]*\\b',
					},
					{
						token: 'keyword.operator',
						regex: '\\+|\\-|\\/|\\/\\/|%|<@>|@>|<@|&|\\^|~|<|>|<=|=>|==|!=|<>|=',
					},
					{
						token: 'paren.lparen',
						regex: '[\\(]',
					},
					{
						token: 'paren.rparen',
						regex: '[\\)]',
					},
					{
						token: 'text',
						regex: '\\s+',
					},
				],
			};
			this.normalizeRules();
		};

		oop.inherits(GwqlHighlightRules, TextHighlightRules);

		exports.GwqlHighlightRules = GwqlHighlightRules;
	},
);
